@@ -462,7 +462,7 @@ public:
 
 bool DiscoverObject::quitWhenIdle()
 {
-    if (!ResourcesModel::global()->isBusy()) {
+    if (!isBusy()) {
         return true;
     }
 
@@ -497,15 +497,27 @@ void DiscoverObject::restore()
         disconnect(TransactionModel::global(), &TransactionModel::countChanged, this, &DiscoverObject::reconsiderQuit);
         disconnect(m_sni.get(), &KStatusNotifierItem::activateRequested, this, &DiscoverObject::restore);
         m_sni.reset();
+        // Set window to topmost one when using SNI
+        if (m_mainWindow) {
+            m_mainWindow->show();
+            m_mainWindow->raise();
+            return;
+        }
     }
+    // Otherwise, just alert the user
     if (m_mainWindow) {
-        m_mainWindow->show();
+        m_mainWindow->alert(0);
     }
 }
 
+bool DiscoverObject::isBusy() const
+{
+    return TransactionModel::global()->rowCount() > 0;
+}
+
 void DiscoverObject::reconsiderQuit()
 {
-    if (ResourcesModel::global()->isBusy()) {
+    if (isBusy()) {
         return;
     }
 
@@ -629,13 +641,13 @@ void DiscoverObject::showError(const QString &msg)
     });
 }
 
-void DiscoverObject::copyTextToClipboard(const QString text)
+void DiscoverObject::copyTextToClipboard(const QString &text)
 {
     QClipboard *clipboard = QGuiApplication::clipboard();
     clipboard->setText(text);
 }
 
-QUrl DiscoverObject::searchUrl(const QString searchText)
+QUrl DiscoverObject::searchUrl(const QString &searchText)
 {
     KUriFilterData filterData(searchText);
     if (KUriFilter::self()->filterSearchUri(filterData, KUriFilter::NormalTextFilter)) {
@@ -644,22 +656,23 @@ QUrl DiscoverObject::searchUrl(const QString searchText)
     return {};
 }
 
-void DiscoverObject::setAboutToPowerOff()
+QString DiscoverObject::mimeTypeComment(const QString &mimeTypeName)
 {
-    for (auto backend : ResourcesModel::global()->backends()) {
-        backend->aboutTo(AbstractResourcesBackend::PowerOff);
+    QMimeDatabase db;
+    QMimeType mimeType = db.mimeTypeForName(mimeTypeName);
+    if (mimeType.isValid()) {
+        return mimeType.comment();
     }
+    return mimeTypeName;
 }
 
-void DiscoverObject::setAboutToReboot()
-{
-    for (auto backend : ResourcesModel::global()->backends()) {
-        backend->aboutTo(AbstractResourcesBackend::Reboot);
-    }
-}
+Q_GLOBAL_STATIC_WITH_ARGS(const bool, s_weAreOnPlasma, (qgetenv("XDG_CURRENT_DESKTOP") == "KDE"))
 
 void DiscoverObject::promptReboot()
 {
+    if (!s_weAreOnPlasma) {
+        qCWarning(DISCOVER_LOG) << "Cannot prompt for reboot outside of Plasma";
+    }
     auto method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.LogoutPrompt"),
                                                  QStringLiteral("/LogoutPrompt"),
                                                  QStringLiteral("org.kde.LogoutPrompt"),
@@ -669,32 +682,56 @@ void DiscoverObject::promptReboot()
 
 void DiscoverObject::rebootNow()
 {
-    setAboutToReboot();
-    auto method = QDBusMessage::createMethodCall(QStringLiteral("org.freedesktop.login1"),
-                                                 QStringLiteral("/org/freedesktop/login1"),
-                                                 QStringLiteral("org.freedesktop.login1.Manager"),
-                                                 QStringLiteral("Reboot"));
-    method.setArguments({true /*interactive*/});
-    QDBusConnection::systemBus().asyncCall(method);
+    const auto backends = ResourcesModel::global()->backends();
+    for (auto backend : backends) {
+        backend->aboutTo(AbstractResourcesBackend::Reboot);
+    }
+
+    QDBusMessage method;
+    if (*s_weAreOnPlasma) {
+        method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.Shutdown"),
+                                                QStringLiteral("/Shutdown"),
+                                                QStringLiteral("org.kde.Shutdown"),
+                                                QStringLiteral("logoutAndReboot"));
+    } else {
+        method = QDBusMessage::createMethodCall(QStringLiteral("org.freedesktop.login1"),
+                                                QStringLiteral("/org/freedesktop/login1"),
+                                                QStringLiteral("org.freedesktop.login1.Manager"),
+                                                QStringLiteral("Reboot"));
+        method.setArguments({true /*interactive*/});
+    }
+    QDBusConnection::sessionBus().asyncCall(method);
 }
 
 void DiscoverObject::shutdownNow()
 {
     bool shouldRebootFirst = false;
-    for (auto backend : ResourcesModel::global()->backends()) {
+    const auto backends = ResourcesModel::global()->backends();
+    for (auto backend : backends) {
         if (backend->needsRebootForPowerOffAction()) {
             shouldRebootFirst = true;
             break;
         }
     }
-    setAboutToPowerOff();
 
-    auto method = QDBusMessage::createMethodCall(QStringLiteral("org.freedesktop.login1"),
-                                                 QStringLiteral("/org/freedesktop/login1"),
-                                                 QStringLiteral("org.freedesktop.login1.Manager"),
-                                                 shouldRebootFirst ? QStringLiteral("Reboot") : QStringLiteral("PowerOff"));
-    method.setArguments({true /*interactive*/});
-    QDBusConnection::systemBus().asyncCall(method);
+    for (auto backend : backends) {
+        backend->aboutTo(AbstractResourcesBackend::PowerOff);
+    }
+
+    QDBusMessage method;
+    if (*s_weAreOnPlasma) {
+        method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.Shutdown"),
+                                                QStringLiteral("/Shutdown"),
+                                                QStringLiteral("org.kde.Shutdown"),
+                                                shouldRebootFirst ? QStringLiteral("logoutAndReboot") : QStringLiteral("logoutAndShutdown"));
+    } else {
+        method = QDBusMessage::createMethodCall(QStringLiteral("org.freedesktop.login1"),
+                                                QStringLiteral("/org/freedesktop/login1"),
+                                                QStringLiteral("org.freedesktop.login1.Manager"),
+                                                shouldRebootFirst ? QStringLiteral("Reboot") : QStringLiteral("PowerOff"));
+        method.setArguments({true /*interactive*/});
+    }
+    QDBusConnection::sessionBus().asyncCall(method);
 }
 
 QString DiscoverObject::describeSources() const
